"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
var graphql_1 = require("graphql");
var logger_1 = require("./logger");
;
exports.getMongoDbProjection = logger_1.logOnError(function (info, graphQLFieldsType) {
    var excludedFields = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        excludedFields[_i - 2] = arguments[_i];
    }
    if (!Object.keys(info).includes('fieldNodes'))
        throw 'First argument of "getMongoDbProjection" must be a GraphQLResolveInfo';
    if (!graphql_1.isType(graphQLFieldsType))
        throw 'Second argument of "getMongoDbProjection" must be a GraphQLType';
    var nodes = common_1.flatten(info.fieldNodes.map(function (_) { return _.selectionSet.selections.slice(); }));
    var projection = getSelectedProjection(nodes, graphQLFieldsType, { info: info, fragments: {} });
    return omitRedundantProjection(projection);
});
function getSelectedProjection(selectionNodes, graphQLFieldsType, extra) {
    var excludedFields = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        excludedFields[_i - 3] = arguments[_i];
    }
    var fields = graphQLFieldsType.getFields();
    return selectionNodes.reduce(function (projection, node) {
        var _a;
        if (node.kind === 'Field') {
            if (node.name.value === '__typename' || excludedFields.includes(node.name.value))
                return projection;
            var field = fields[node.name.value];
            if (!!field.resolve) {
                var dependencies = field["dependencies"] || [];
                var dependenciesProjection = dependencies.reduce(function (agg, dependency) {
                    var _a;
                    return (__assign({}, agg, (_a = {}, _a[dependency] = 1, _a)));
                }, {});
                return __assign({}, projection, dependenciesProjection);
            }
            if (!node.selectionSet)
                return __assign({}, projection, (_a = {}, _a[node.name.value] = 1, _a));
            var nested = getSelectedProjection(node.selectionSet.selections.slice(), common_1.getInnerType(field.type), extra);
            return __assign({}, projection, common_1.addPrefixToProperties(nested, node.name.value + "."));
        }
        else if (node.kind === 'InlineFragment') {
            var type = extra.info.schema.getType(node.typeCondition.name.value);
            return __assign({}, projection, getSelectedProjection.apply(void 0, [node.selectionSet.selections.slice(), type, extra].concat(excludedFields)));
        }
        else if (node.kind === 'FragmentSpread') {
            return __assign({}, projection, getFragmentProjection(node, graphQLFieldsType, extra));
        }
    }, {});
}
function getFragmentProjection(fragmentSpreadNode, graphQLFieldsType, extra) {
    var fragmentName = fragmentSpreadNode.name.value;
    if (extra.fragments[fragmentName])
        return extra.fragments[fragmentName];
    var fragmentNode = extra.info.fragments[fragmentName];
    extra.fragments[fragmentName] = getSelectedProjection(fragmentNode.selectionSet.selections.slice(), graphQLFieldsType, extra);
    return extra.fragments[fragmentName];
}
function omitRedundantProjection(projection) {
    return Object.keys(projection).reduce(function (proj, key) {
        var _a;
        if (Object.keys(projection).some(function (otherKey) {
            return otherKey !== key && new RegExp("^" + otherKey + "[.]").test(key);
        })) {
            return proj;
        }
        return __assign({}, proj, (_a = {}, _a[key] = 1, _a));
    }, {});
}
